"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .resourcetypeenum import ResourceTypeEnum
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class ResourceToPublishDtoTypedDict(TypedDict):
    resource_type: ResourceTypeEnum
    r"""Type of the layout"""
    resource_id: str
    r"""Unique identifier of the resource to publish"""


class ResourceToPublishDto(BaseModel):
    resource_type: Annotated[ResourceTypeEnum, pydantic.Field(alias="resourceType")]
    r"""Type of the layout"""

    resource_id: Annotated[str, pydantic.Field(alias="resourceId")]
    r"""Unique identifier of the resource to publish"""


try:
    ResourceToPublishDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .notificationtriggerdto import (
    NotificationTriggerDto,
    NotificationTriggerDtoTypedDict,
)
from .resourceoriginenum import ResourceOriginEnum
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ActivityNotificationTemplateResponseDtoTypedDict(TypedDict):
    name: str
    r"""Name of the template"""
    triggers: List[NotificationTriggerDtoTypedDict]
    r"""Triggers of the template"""
    id: NotRequired[str]
    r"""Unique identifier of the template"""
    origin: NotRequired[ResourceOriginEnum]
    r"""Origin of the layout"""


class ActivityNotificationTemplateResponseDto(BaseModel):
    name: str
    r"""Name of the template"""

    triggers: List[NotificationTriggerDto]
    r"""Triggers of the template"""

    id: Annotated[Optional[str], pydantic.Field(alias="_id")] = None
    r"""Unique identifier of the template"""

    origin: Optional[ResourceOriginEnum] = None
    r"""Origin of the layout"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["_id", "origin"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    ActivityNotificationTemplateResponseDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .smscontroldto import SmsControlDto, SmsControlDtoTypedDict
from .uischema import UISchema, UISchemaTypedDict
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SmsControlsMetadataResponseDtoTypedDict(TypedDict):
    values: SmsControlDtoTypedDict
    r"""Control values specific to SMS"""
    data_schema: NotRequired[Dict[str, Any]]
    r"""JSON Schema for data"""
    ui_schema: NotRequired[UISchemaTypedDict]
    r"""UI Schema for rendering"""


class SmsControlsMetadataResponseDto(BaseModel):
    values: SmsControlDto
    r"""Control values specific to SMS"""

    data_schema: Annotated[
        Optional[Dict[str, Any]], pydantic.Field(alias="dataSchema")
    ] = None
    r"""JSON Schema for data"""

    ui_schema: Annotated[Optional[UISchema], pydantic.Field(alias="uiSchema")] = None
    r"""UI Schema for rendering"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["dataSchema", "uiSchema"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    SmsControlsMetadataResponseDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .environmentdiffsummarydto import (
    EnvironmentDiffSummaryDto,
    EnvironmentDiffSummaryDtoTypedDict,
)
from .resourcediffresultdto import ResourceDiffResultDto, ResourceDiffResultDtoTypedDict
from novu_py.types import BaseModel
import pydantic
from typing import List
from typing_extensions import Annotated, TypedDict


class DiffEnvironmentResponseDtoTypedDict(TypedDict):
    source_environment_id: str
    r"""Source environment ID"""
    target_environment_id: str
    r"""Target environment ID"""
    resources: List[ResourceDiffResultDtoTypedDict]
    r"""Diff resources by resource type"""
    summary: EnvironmentDiffSummaryDtoTypedDict
    r"""Overall summary"""


class DiffEnvironmentResponseDto(BaseModel):
    source_environment_id: Annotated[str, pydantic.Field(alias="sourceEnvironmentId")]
    r"""Source environment ID"""

    target_environment_id: Annotated[str, pydantic.Field(alias="targetEnvironmentId")]
    r"""Target environment ID"""

    resources: List[ResourceDiffResultDto]
    r"""Diff resources by resource type"""

    summary: EnvironmentDiffSummaryDto
    r"""Overall summary"""


try:
    DiffEnvironmentResponseDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class StepsOverridesTypedDict(TypedDict):
    providers: NotRequired[Dict[str, Dict[str, Any]]]
    r"""Passing the provider id and the provider specific configurations"""
    layout_id: NotRequired[Nullable[str]]
    r"""Override the or remove the layout for this specific step"""


class StepsOverrides(BaseModel):
    providers: Optional[Dict[str, Dict[str, Any]]] = None
    r"""Passing the provider id and the provider specific configurations"""

    layout_id: Annotated[OptionalNullable[str], pydantic.Field(alias="layoutId")] = (
        UNSET
    )
    r"""Override the or remove the layout for this specific step"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["providers", "layoutId"])
        nullable_fields = set(["layoutId"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    StepsOverrides.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .layoutcreationsourceenum import LayoutCreationSourceEnum
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateLayoutDtoTypedDict(TypedDict):
    layout_id: str
    r"""Unique identifier for the layout"""
    name: str
    r"""Name of the layout"""
    is_translation_enabled: NotRequired[bool]
    r"""Enable or disable translations for this layout"""
    source: NotRequired[LayoutCreationSourceEnum]
    r"""Source of layout creation"""


class CreateLayoutDto(BaseModel):
    layout_id: Annotated[str, pydantic.Field(alias="layoutId")]
    r"""Unique identifier for the layout"""

    name: str
    r"""Name of the layout"""

    is_translation_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="isTranslationEnabled")
    ] = False
    r"""Enable or disable translations for this layout"""

    source: Annotated[
        Optional[LayoutCreationSourceEnum], pydantic.Field(alias="__source")
    ] = LayoutCreationSourceEnum.DASHBOARD
    r"""Source of layout creation"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["isTranslationEnabled", "__source"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreateLayoutDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing_extensions import TypedDict


class PublishSummaryDtoTypedDict(TypedDict):
    resources: float
    r"""Number of resources processed"""
    successful: float
    r"""Number of successful syncs"""
    failed: float
    r"""Number of failed syncs"""
    skipped: float
    r"""Number of skipped resources"""


class PublishSummaryDto(BaseModel):
    resources: float
    r"""Number of resources processed"""

    successful: float
    r"""Number of successful syncs"""

    failed: float
    r"""Number of failed syncs"""

    skipped: float
    r"""Number of skipped resources"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .channelcredentials import ChannelCredentials, ChannelCredentialsTypedDict
from .chatorpushproviderenum import ChatOrPushProviderEnum
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UpdateSubscriberChannelRequestDtoTypedDict(TypedDict):
    provider_id: ChatOrPushProviderEnum
    r"""The provider identifier for the credentials"""
    credentials: ChannelCredentialsTypedDict
    r"""Credentials payload for the specified provider"""
    integration_identifier: NotRequired[str]
    r"""The integration identifier"""


class UpdateSubscriberChannelRequestDto(BaseModel):
    provider_id: Annotated[ChatOrPushProviderEnum, pydantic.Field(alias="providerId")]
    r"""The provider identifier for the credentials"""

    credentials: ChannelCredentials
    r"""Credentials payload for the specified provider"""

    integration_identifier: Annotated[
        Optional[str], pydantic.Field(alias="integrationIdentifier")
    ] = None
    r"""The integration identifier"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["integrationIdentifier"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    UpdateSubscriberChannelRequestDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .diffactionenum import DiffActionEnum
from .resourcetypeenum import ResourceTypeEnum
from datetime import datetime
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ResourceDiffDtoUpdatedByTypedDict(TypedDict):
    r"""User who last updated the resource"""

    id: str
    r"""User ID"""
    first_name: str
    r"""User first name"""
    last_name: NotRequired[Nullable[str]]
    r"""User last name"""
    external_id: NotRequired[str]
    r"""User external ID"""


class ResourceDiffDtoUpdatedBy(BaseModel):
    r"""User who last updated the resource"""

    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""User ID"""

    first_name: Annotated[str, pydantic.Field(alias="firstName")]
    r"""User first name"""

    last_name: Annotated[OptionalNullable[str], pydantic.Field(alias="lastName")] = (
        UNSET
    )
    r"""User last name"""

    external_id: Annotated[Optional[str], pydantic.Field(alias="externalId")] = None
    r"""User external ID"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["lastName", "externalId"])
        nullable_fields = set(["lastName"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class ResourceDiffDtoSourceResourceTypedDict(TypedDict):
    r"""Source resource information"""

    id: NotRequired[Nullable[str]]
    r"""Resource ID (workflow ID or step ID)"""
    name: NotRequired[Nullable[str]]
    r"""Resource name (workflow name or step name)"""
    updated_by: NotRequired[Nullable[ResourceDiffDtoUpdatedByTypedDict]]
    r"""User who last updated the resource"""
    updated_at: NotRequired[Nullable[datetime]]
    r"""When the resource was last updated"""


class ResourceDiffDtoSourceResource(BaseModel):
    r"""Source resource information"""

    id: OptionalNullable[str] = UNSET
    r"""Resource ID (workflow ID or step ID)"""

    name: OptionalNullable[str] = UNSET
    r"""Resource name (workflow name or step name)"""

    updated_by: Annotated[
        OptionalNullable[ResourceDiffDtoUpdatedBy], pydantic.Field(alias="updatedBy")
    ] = UNSET
    r"""User who last updated the resource"""

    updated_at: Annotated[
        OptionalNullable[datetime], pydantic.Field(alias="updatedAt")
    ] = UNSET
    r"""When the resource was last updated"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["id", "name", "updatedBy", "updatedAt"])
        nullable_fields = set(["id", "name", "updatedBy", "updatedAt"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class ResourceDiffDtoTargetResourceUpdatedByTypedDict(TypedDict):
    r"""User who last updated the resource"""

    id: str
    r"""User ID"""
    first_name: str
    r"""User first name"""
    last_name: NotRequired[Nullable[str]]
    r"""User last name"""
    external_id: NotRequired[str]
    r"""User external ID"""


class ResourceDiffDtoTargetResourceUpdatedBy(BaseModel):
    r"""User who last updated the resource"""

    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""User ID"""

    first_name: Annotated[str, pydantic.Field(alias="firstName")]
    r"""User first name"""

    last_name: Annotated[OptionalNullable[str], pydantic.Field(alias="lastName")] = (
        UNSET
    )
    r"""User last name"""

    external_id: Annotated[Optional[str], pydantic.Field(alias="externalId")] = None
    r"""User external ID"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["lastName", "externalId"])
        nullable_fields = set(["lastName"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class ResourceDiffDtoTargetResourceTypedDict(TypedDict):
    r"""Target resource information"""

    id: NotRequired[Nullable[str]]
    r"""Resource ID (workflow ID or step ID)"""
    name: NotRequired[Nullable[str]]
    r"""Resource name (workflow name or step name)"""
    updated_by: NotRequired[Nullable[ResourceDiffDtoTargetResourceUpdatedByTypedDict]]
    r"""User who last updated the resource"""
    updated_at: NotRequired[Nullable[datetime]]
    r"""When the resource was last updated"""


class ResourceDiffDtoTargetResource(BaseModel):
    r"""Target resource information"""

    id: OptionalNullable[str] = UNSET
    r"""Resource ID (workflow ID or step ID)"""

    name: OptionalNullable[str] = UNSET
    r"""Resource name (workflow name or step name)"""

    updated_by: Annotated[
        OptionalNullable[ResourceDiffDtoTargetResourceUpdatedBy],
        pydantic.Field(alias="updatedBy"),
    ] = UNSET
    r"""User who last updated the resource"""

    updated_at: Annotated[
        OptionalNullable[datetime], pydantic.Field(alias="updatedAt")
    ] = UNSET
    r"""When the resource was last updated"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["id", "name", "updatedBy", "updatedAt"])
        nullable_fields = set(["id", "name", "updatedBy", "updatedAt"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class DiffsTypedDict(TypedDict):
    r"""Detailed changes (only for modified resources)"""

    previous: NotRequired[Nullable[Dict[str, Any]]]
    r"""Previous state of the resource (null for added resources)"""
    new: NotRequired[Nullable[Dict[str, Any]]]
    r"""New state of the resource (null for deleted resources)"""


class Diffs(BaseModel):
    r"""Detailed changes (only for modified resources)"""

    previous: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Previous state of the resource (null for added resources)"""

    new: OptionalNullable[Dict[str, Any]] = UNSET
    r"""New state of the resource (null for deleted resources)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["previous", "new"])
        nullable_fields = set(["previous", "new"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class ResourceDiffDtoTypedDict(TypedDict):
    resource_type: ResourceTypeEnum
    r"""Type of the layout"""
    action: DiffActionEnum
    r"""Type of change"""
    source_resource: NotRequired[Nullable[ResourceDiffDtoSourceResourceTypedDict]]
    r"""Source resource information"""
    target_resource: NotRequired[Nullable[ResourceDiffDtoTargetResourceTypedDict]]
    r"""Target resource information"""
    diffs: NotRequired[DiffsTypedDict]
    r"""Detailed changes (only for modified resources)"""
    step_type: NotRequired[str]
    r"""Step type (only for step resources)"""
    previous_index: NotRequired[float]
    r"""Previous index in steps array (for moved/deleted steps)"""
    new_index: NotRequired[float]
    r"""New index in steps array (for moved/added steps)"""


class ResourceDiffDto(BaseModel):
    resource_type: Annotated[ResourceTypeEnum, pydantic.Field(alias="resourceType")]
    r"""Type of the layout"""

    action: DiffActionEnum
    r"""Type of change"""

    source_resource: Annotated[
        OptionalNullable[ResourceDiffDtoSourceResource],
        pydantic.Field(alias="sourceResource"),
    ] = UNSET
    r"""Source resource information"""

    target_resource: Annotated[
        OptionalNullable[ResourceDiffDtoTargetResource],
        pydantic.Field(alias="targetResource"),
    ] = UNSET
    r"""Target resource information"""

    diffs: Optional[Diffs] = None
    r"""Detailed changes (only for modified resources)"""

    step_type: Annotated[Optional[str], pydantic.Field(alias="stepType")] = None
    r"""Step type (only for step resources)"""

    previous_index: Annotated[
        Optional[float], pydantic.Field(alias="previousIndex")
    ] = None
    r"""Previous index in steps array (for moved/deleted steps)"""

    new_index: Annotated[Optional[float], pydantic.Field(alias="newIndex")] = None
    r"""New index in steps array (for moved/added steps)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "sourceResource",
                "targetResource",
                "diffs",
                "stepType",
                "previousIndex",
                "newIndex",
            ]
        )
        nullable_fields = set(["sourceResource", "targetResource"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    ResourceDiffDtoUpdatedBy.model_rebuild()
except NameError:
    pass
try:
    ResourceDiffDtoSourceResource.model_rebuild()
except NameError:
    pass
try:
    ResourceDiffDtoTargetResourceUpdatedBy.model_rebuild()
except NameError:
    pass
try:
    ResourceDiffDtoTargetResource.model_rebuild()
except NameError:
    pass
try:
    ResourceDiffDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .channeltypeenum import ChannelTypeEnum
from .messagesresponsedto import MessagesResponseDto, MessagesResponseDtoTypedDict
from novu_py.types import BaseModel, UNSET_SENTINEL
from novu_py.utils import FieldMetadata, HeaderMetadata, QueryParamMetadata
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class MessagesControllerGetMessagesRequestTypedDict(TypedDict):
    channel: NotRequired[ChannelTypeEnum]
    r"""Channel type through which the message is sent"""
    subscriber_id: NotRequired[str]
    transaction_id: NotRequired[List[str]]
    context_keys: NotRequired[List[str]]
    r"""Filter by exact context keys, order insensitive (format: \"type:id\")"""
    page: NotRequired[float]
    limit: NotRequired[float]
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class MessagesControllerGetMessagesRequest(BaseModel):
    channel: Annotated[
        Optional[ChannelTypeEnum],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Channel type through which the message is sent"""

    subscriber_id: Annotated[
        Optional[str],
        pydantic.Field(alias="subscriberId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    transaction_id: Annotated[
        Optional[List[str]],
        pydantic.Field(alias="transactionId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    context_keys: Annotated[
        Optional[List[str]],
        pydantic.Field(alias="contextKeys"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter by exact context keys, order insensitive (format: \"type:id\")"""

    page: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 0

    limit: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "channel",
                "subscriberId",
                "transactionId",
                "contextKeys",
                "page",
                "limit",
                "idempotency-key",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class MessagesControllerGetMessagesResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: MessagesResponseDtoTypedDict


class MessagesControllerGetMessagesResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: MessagesResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .resourcetopublishdto import ResourceToPublishDto, ResourceToPublishDtoTypedDict
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PublishEnvironmentRequestDtoTypedDict(TypedDict):
    source_environment_id: NotRequired[str]
    r"""Source environment ID to sync from. Defaults to the Development environment if not provided."""
    dry_run: NotRequired[bool]
    r"""Perform a dry run without making actual changes"""
    resources: NotRequired[List[ResourceToPublishDtoTypedDict]]
    r"""Array of specific resources to publish. If not provided, all resources will be published."""


class PublishEnvironmentRequestDto(BaseModel):
    source_environment_id: Annotated[
        Optional[str], pydantic.Field(alias="sourceEnvironmentId")
    ] = None
    r"""Source environment ID to sync from. Defaults to the Development environment if not provided."""

    dry_run: Annotated[Optional[bool], pydantic.Field(alias="dryRun")] = False
    r"""Perform a dry run without making actual changes"""

    resources: Optional[List[ResourceToPublishDto]] = None
    r"""Array of specific resources to publish. If not provided, all resources will be published."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["sourceEnvironmentId", "dryRun", "resources"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    PublishEnvironmentRequestDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ChannelCredentialsDtoTypedDict(TypedDict):
    webhook_url: NotRequired[str]
    r"""The URL for the webhook associated with the channel."""
    device_tokens: NotRequired[List[str]]
    r"""An array of device tokens for push notifications."""


class ChannelCredentialsDto(BaseModel):
    webhook_url: Annotated[Optional[str], pydantic.Field(alias="webhookUrl")] = None
    r"""The URL for the webhook associated with the channel."""

    device_tokens: Annotated[
        Optional[List[str]], pydantic.Field(alias="deviceTokens")
    ] = None
    r"""An array of device tokens for push notifications."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["webhookUrl", "deviceTokens"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    ChannelCredentialsDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing_extensions import TypedDict


class DiffSummaryDtoTypedDict(TypedDict):
    added: float
    r"""Number of added resources (workflows and steps)"""
    modified: float
    r"""Number of modified resources (workflows and steps)"""
    deleted: float
    r"""Number of deleted resources (workflows and steps)"""
    unchanged: float
    r"""Number of unchanged resources (workflows and steps)"""


class DiffSummaryDto(BaseModel):
    added: float
    r"""Number of added resources (workflows and steps)"""

    modified: float
    r"""Number of modified resources (workflows and steps)"""

    deleted: float
    r"""Number of deleted resources (workflows and steps)"""

    unchanged: float
    r"""Number of unchanged resources (workflows and steps)"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .directionenum import DirectionEnum
from .layoutresponsedtosortfield import LayoutResponseDtoSortField
from .listlayoutresponsedto import ListLayoutResponseDto, ListLayoutResponseDtoTypedDict
from novu_py.types import BaseModel, UNSET_SENTINEL
from novu_py.utils import FieldMetadata, HeaderMetadata, QueryParamMetadata
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LayoutsControllerListRequestTypedDict(TypedDict):
    limit: NotRequired[float]
    r"""Number of items to return per page"""
    offset: NotRequired[float]
    r"""Number of items to skip before starting to return results"""
    order_direction: NotRequired[DirectionEnum]
    r"""Direction of sorting"""
    order_by: NotRequired[LayoutResponseDtoSortField]
    r"""Field to sort the results by"""
    query: NotRequired[str]
    r"""Search query to filter layouts"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class LayoutsControllerListRequest(BaseModel):
    limit: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Number of items to return per page"""

    offset: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Number of items to skip before starting to return results"""

    order_direction: Annotated[
        Optional[DirectionEnum],
        pydantic.Field(alias="orderDirection"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Direction of sorting"""

    order_by: Annotated[
        Optional[LayoutResponseDtoSortField],
        pydantic.Field(alias="orderBy"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Field to sort the results by"""

    query: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Search query to filter layouts"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["limit", "offset", "orderDirection", "orderBy", "query", "idempotency-key"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class LayoutsControllerListResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: ListLayoutResponseDtoTypedDict


class LayoutsControllerListResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: ListLayoutResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .triggereventrequestdto import (
    TriggerEventRequestDto,
    TriggerEventRequestDtoTypedDict,
)
from .triggereventresponsedto import (
    TriggerEventResponseDto,
    TriggerEventResponseDtoTypedDict,
)
from novu_py.types import BaseModel, UNSET_SENTINEL
from novu_py.utils import FieldMetadata, HeaderMetadata, RequestMetadata
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EventsControllerTriggerRequestTypedDict(TypedDict):
    trigger_event_request_dto: TriggerEventRequestDtoTypedDict
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class EventsControllerTriggerRequest(BaseModel):
    trigger_event_request_dto: Annotated[
        TriggerEventRequestDto,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["idempotency-key"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class EventsControllerTriggerResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: TriggerEventResponseDtoTypedDict


class EventsControllerTriggerResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: TriggerEventResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .publishsummarydto import PublishSummaryDto, PublishSummaryDtoTypedDict
from .syncresultdto import SyncResultDto, SyncResultDtoTypedDict
from novu_py.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class PublishEnvironmentResponseDtoTypedDict(TypedDict):
    results: List[SyncResultDtoTypedDict]
    r"""Sync results by resource type"""
    summary: PublishSummaryDtoTypedDict
    r"""Summary of the sync operation"""


class PublishEnvironmentResponseDto(BaseModel):
    results: List[SyncResultDto]
    r"""Sync results by resource type"""

    summary: PublishSummaryDto
    r"""Summary of the sync operation"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class PushControlDtoTypedDict(TypedDict):
    skip: NotRequired[Dict[str, Any]]
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""
    subject: NotRequired[str]
    r"""Subject/title of the push notification."""
    body: NotRequired[str]
    r"""Body content of the push notification."""


class PushControlDto(BaseModel):
    skip: Optional[Dict[str, Any]] = None
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""

    subject: Optional[str] = None
    r"""Subject/title of the push notification."""

    body: Optional[str] = None
    r"""Body content of the push notification."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["skip", "subject", "body"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class PatchPreferenceChannelsDtoTypedDict(TypedDict):
    email: NotRequired[bool]
    r"""Email channel preference"""
    sms: NotRequired[bool]
    r"""SMS channel preference"""
    in_app: NotRequired[bool]
    r"""In-app channel preference"""
    push: NotRequired[bool]
    r"""Push channel preference"""
    chat: NotRequired[bool]
    r"""Chat channel preference"""


class PatchPreferenceChannelsDto(BaseModel):
    email: Optional[bool] = None
    r"""Email channel preference"""

    sms: Optional[bool] = None
    r"""SMS channel preference"""

    in_app: Optional[bool] = None
    r"""In-app channel preference"""

    push: Optional[bool] = None
    r"""Push channel preference"""

    chat: Optional[bool] = None
    r"""Chat channel preference"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["email", "sms", "in_app", "push", "chat"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .emailblockstyles import EmailBlockStyles, EmailBlockStylesTypedDict
from .emailblocktypeenum import EmailBlockTypeEnum
from novu_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class EmailBlockTypedDict(TypedDict):
    type: EmailBlockTypeEnum
    r"""Type of the email block"""
    content: str
    r"""Content of the email block"""
    url: NotRequired[str]
    r"""URL associated with the email block, if any"""
    styles: NotRequired[EmailBlockStylesTypedDict]
    r"""Styles applied to the email block"""


class EmailBlock(BaseModel):
    type: EmailBlockTypeEnum
    r"""Type of the email block"""

    content: str
    r"""Content of the email block"""

    url: Optional[str] = None
    r"""URL associated with the email block, if any"""

    styles: Optional[EmailBlockStyles] = None
    r"""Styles applied to the email block"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["url", "styles"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .resourcetypeenum import ResourceTypeEnum
from .syncactionenum import SyncActionEnum
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class SyncedWorkflowDtoTypedDict(TypedDict):
    resource_type: ResourceTypeEnum
    r"""Type of the layout"""
    resource_id: str
    r"""Resource ID"""
    resource_name: str
    r"""Resource name"""
    action: SyncActionEnum
    r"""Sync action performed"""


class SyncedWorkflowDto(BaseModel):
    resource_type: Annotated[ResourceTypeEnum, pydantic.Field(alias="resourceType")]
    r"""Type of the layout"""

    resource_id: Annotated[str, pydantic.Field(alias="resourceId")]
    r"""Resource ID"""

    resource_name: Annotated[str, pydantic.Field(alias="resourceName")]
    r"""Resource name"""

    action: SyncActionEnum
    r"""Sync action performed"""


try:
    SyncedWorkflowDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .uischemagroupenum import UISchemaGroupEnum
from .uischemaproperty import UISchemaProperty, UISchemaPropertyTypedDict
from novu_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class UISchemaTypedDict(TypedDict):
    group: NotRequired[UISchemaGroupEnum]
    r"""Group of the UI Schema"""
    properties: NotRequired[Dict[str, UISchemaPropertyTypedDict]]
    r"""Properties of the UI Schema"""


class UISchema(BaseModel):
    group: Optional[UISchemaGroupEnum] = None
    r"""Group of the UI Schema"""

    properties: Optional[Dict[str, UISchemaProperty]] = None
    r"""Properties of the UI Schema"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["group", "properties"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .failedworkflowdto import FailedWorkflowDto, FailedWorkflowDtoTypedDict
from .resourcetypeenum import ResourceTypeEnum
from .skippedworkflowdto import SkippedWorkflowDto, SkippedWorkflowDtoTypedDict
from .syncedworkflowdto import SyncedWorkflowDto, SyncedWorkflowDtoTypedDict
from novu_py.types import BaseModel
import pydantic
from typing import List
from typing_extensions import Annotated, TypedDict


class SyncResultDtoTypedDict(TypedDict):
    resource_type: ResourceTypeEnum
    r"""Type of the layout"""
    successful: List[SyncedWorkflowDtoTypedDict]
    r"""Successfully synced resources"""
    failed: List[FailedWorkflowDtoTypedDict]
    r"""Failed resource syncs"""
    skipped: List[SkippedWorkflowDtoTypedDict]
    r"""Skipped resources"""
    total_processed: float
    r"""Total number of resources processed"""


class SyncResultDto(BaseModel):
    resource_type: Annotated[ResourceTypeEnum, pydantic.Field(alias="resourceType")]
    r"""Type of the layout"""

    successful: List[SyncedWorkflowDto]
    r"""Successfully synced resources"""

    failed: List[FailedWorkflowDto]
    r"""Failed resource syncs"""

    skipped: List[SkippedWorkflowDto]
    r"""Skipped resources"""

    total_processed: Annotated[float, pydantic.Field(alias="totalProcessed")]
    r"""Total number of resources processed"""


try:
    SyncResultDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .bulkupdatesubscriberpreferenceitemdto import (
    BulkUpdateSubscriberPreferenceItemDto,
    BulkUpdateSubscriberPreferenceItemDtoTypedDict,
)
from novu_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, List, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


class Context2TypedDict(TypedDict):
    r"""Rich context object with id and optional data"""

    id: str
    data: NotRequired[Dict[str, Any]]
    r"""Optional additional context data"""


class Context2(BaseModel):
    r"""Rich context object with id and optional data"""

    id: str

    data: Optional[Dict[str, Any]] = None
    r"""Optional additional context data"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["data"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


BulkUpdateSubscriberPreferencesDtoContextTypedDict = TypeAliasType(
    "BulkUpdateSubscriberPreferencesDtoContextTypedDict", Union[Context2TypedDict, str]
)


BulkUpdateSubscriberPreferencesDtoContext = TypeAliasType(
    "BulkUpdateSubscriberPreferencesDtoContext", Union[Context2, str]
)


class BulkUpdateSubscriberPreferencesDtoTypedDict(TypedDict):
    preferences: List[BulkUpdateSubscriberPreferenceItemDtoTypedDict]
    r"""Array of workflow preferences to update (maximum 100 items)"""
    context: NotRequired[Dict[str, BulkUpdateSubscriberPreferencesDtoContextTypedDict]]


class BulkUpdateSubscriberPreferencesDto(BaseModel):
    preferences: List[BulkUpdateSubscriberPreferenceItemDto]
    r"""Array of workflow preferences to update (maximum 100 items)"""

    context: Optional[Dict[str, BulkUpdateSubscriberPreferencesDtoContext]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["context"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .metadto import MetaDto, MetaDtoTypedDict
from .subscriptionerrordto import SubscriptionErrorDto, SubscriptionErrorDtoTypedDict
from .subscriptionresponsedto import (
    SubscriptionResponseDto,
    SubscriptionResponseDtoTypedDict,
)
from novu_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class CreateSubscriptionsResponseDtoTypedDict(TypedDict):
    data: List[SubscriptionResponseDtoTypedDict]
    r"""The list of successfully created subscriptions"""
    meta: MetaDtoTypedDict
    r"""Metadata about the operation"""
    errors: NotRequired[List[SubscriptionErrorDtoTypedDict]]
    r"""The list of errors for failed subscription attempts"""


class CreateSubscriptionsResponseDto(BaseModel):
    data: List[SubscriptionResponseDto]
    r"""The list of successfully created subscriptions"""

    meta: MetaDto
    r"""Metadata about the operation"""

    errors: Optional[List[SubscriptionErrorDto]] = None
    r"""The list of errors for failed subscription attempts"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["errors"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .buttontypeenum import ButtonTypeEnum
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class MessageButtonTypedDict(TypedDict):
    type: ButtonTypeEnum
    r"""Type of button for the action result"""
    content: str
    r"""Content of the button"""
    result_content: NotRequired[str]
    r"""Content of the result when the button is clicked"""


class MessageButton(BaseModel):
    type: ButtonTypeEnum
    r"""Type of button for the action result"""

    content: str
    r"""Content of the button"""

    result_content: Annotated[Optional[str], pydantic.Field(alias="resultContent")] = (
        None
    )
    r"""Content of the result when the button is clicked"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["resultContent"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    MessageButton.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriberpreferencechannels import (
    SubscriberPreferenceChannels,
    SubscriberPreferenceChannelsTypedDict,
)
from .subscriberpreferenceoverridedto import (
    SubscriberPreferenceOverrideDto,
    SubscriberPreferenceOverrideDtoTypedDict,
)
from .subscriberpreferencesworkflowinfodto import (
    SubscriberPreferencesWorkflowInfoDto,
    SubscriberPreferencesWorkflowInfoDtoTypedDict,
)
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SubscriberWorkflowPreferenceDtoTypedDict(TypedDict):
    enabled: bool
    r"""Whether notifications are enabled for this workflow"""
    channels: SubscriberPreferenceChannelsTypedDict
    r"""Channel-specific preference settings for this workflow"""
    overrides: List[SubscriberPreferenceOverrideDtoTypedDict]
    r"""List of preference overrides"""
    workflow: SubscriberPreferencesWorkflowInfoDtoTypedDict
    r"""Workflow information"""
    updated_at: NotRequired[str]
    r"""Timestamp when the subscriber last updated their preference. Only present if subscriber explicitly set preferences."""


class SubscriberWorkflowPreferenceDto(BaseModel):
    enabled: bool
    r"""Whether notifications are enabled for this workflow"""

    channels: SubscriberPreferenceChannels
    r"""Channel-specific preference settings for this workflow"""

    overrides: List[SubscriberPreferenceOverrideDto]
    r"""List of preference overrides"""

    workflow: SubscriberPreferencesWorkflowInfoDto
    r"""Workflow information"""

    updated_at: Annotated[Optional[str], pydantic.Field(alias="updatedAt")] = None
    r"""Timestamp when the subscriber last updated their preference. Only present if subscriber explicitly set preferences."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["updatedAt"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    SubscriberWorkflowPreferenceDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .resourcetypeenum import ResourceTypeEnum
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class FailedWorkflowDtoTypedDict(TypedDict):
    resource_type: ResourceTypeEnum
    r"""Type of the layout"""
    resource_id: str
    r"""Resource ID"""
    resource_name: str
    r"""Resource name"""
    error: str
    r"""Error message"""
    stack: NotRequired[str]
    r"""Error stack trace"""


class FailedWorkflowDto(BaseModel):
    resource_type: Annotated[ResourceTypeEnum, pydantic.Field(alias="resourceType")]
    r"""Type of the layout"""

    resource_id: Annotated[str, pydantic.Field(alias="resourceId")]
    r"""Resource ID"""

    resource_name: Annotated[str, pydantic.Field(alias="resourceName")]
    r"""Resource name"""

    error: str
    r"""Error message"""

    stack: Optional[str] = None
    r"""Error stack trace"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["stack"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    FailedWorkflowDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .getsubscriberpreferencesdto import (
    GetSubscriberPreferencesDto,
    GetSubscriberPreferencesDtoTypedDict,
)
from enum import Enum
from novu_py.types import BaseModel, UNSET_SENTINEL
from novu_py.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class Criticality(str, Enum):
    CRITICAL = "critical"
    NON_CRITICAL = "nonCritical"
    ALL = "all"


class SubscribersControllerGetSubscriberPreferencesRequestTypedDict(TypedDict):
    subscriber_id: str
    criticality: NotRequired[Criticality]
    context_keys: NotRequired[List[str]]
    r"""Context keys for filtering preferences (e.g., [\"tenant:acme\"])"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class SubscribersControllerGetSubscriberPreferencesRequest(BaseModel):
    subscriber_id: Annotated[
        str,
        pydantic.Field(alias="subscriberId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    criticality: Annotated[
        Optional[Criticality],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = Criticality.NON_CRITICAL

    context_keys: Annotated[
        Optional[List[str]],
        pydantic.Field(alias="contextKeys"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Context keys for filtering preferences (e.g., [\"tenant:acme\"])"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["criticality", "contextKeys", "idempotency-key"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class SubscribersControllerGetSubscriberPreferencesResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: GetSubscriberPreferencesDtoTypedDict


class SubscribersControllerGetSubscriberPreferencesResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: GetSubscriberPreferencesDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .timerangedto import TimeRangeDto, TimeRangeDtoTypedDict
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class MondayTypedDict(TypedDict):
    r"""Monday schedule"""

    is_enabled: bool
    r"""Day schedule enabled"""
    hours: NotRequired[List[TimeRangeDtoTypedDict]]
    r"""Hours"""


class Monday(BaseModel):
    r"""Monday schedule"""

    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Day schedule enabled"""

    hours: Optional[List[TimeRangeDto]] = None
    r"""Hours"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["hours"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class TuesdayTypedDict(TypedDict):
    r"""Tuesday schedule"""

    is_enabled: bool
    r"""Day schedule enabled"""
    hours: NotRequired[List[TimeRangeDtoTypedDict]]
    r"""Hours"""


class Tuesday(BaseModel):
    r"""Tuesday schedule"""

    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Day schedule enabled"""

    hours: Optional[List[TimeRangeDto]] = None
    r"""Hours"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["hours"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class WednesdayTypedDict(TypedDict):
    r"""Wednesday schedule"""

    is_enabled: bool
    r"""Day schedule enabled"""
    hours: NotRequired[List[TimeRangeDtoTypedDict]]
    r"""Hours"""


class Wednesday(BaseModel):
    r"""Wednesday schedule"""

    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Day schedule enabled"""

    hours: Optional[List[TimeRangeDto]] = None
    r"""Hours"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["hours"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ThursdayTypedDict(TypedDict):
    r"""Thursday schedule"""

    is_enabled: bool
    r"""Day schedule enabled"""
    hours: NotRequired[List[TimeRangeDtoTypedDict]]
    r"""Hours"""


class Thursday(BaseModel):
    r"""Thursday schedule"""

    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Day schedule enabled"""

    hours: Optional[List[TimeRangeDto]] = None
    r"""Hours"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["hours"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class FridayTypedDict(TypedDict):
    r"""Friday schedule"""

    is_enabled: bool
    r"""Day schedule enabled"""
    hours: NotRequired[List[TimeRangeDtoTypedDict]]
    r"""Hours"""


class Friday(BaseModel):
    r"""Friday schedule"""

    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Day schedule enabled"""

    hours: Optional[List[TimeRangeDto]] = None
    r"""Hours"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["hours"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class SaturdayTypedDict(TypedDict):
    r"""Saturday schedule"""

    is_enabled: bool
    r"""Day schedule enabled"""
    hours: NotRequired[List[TimeRangeDtoTypedDict]]
    r"""Hours"""


class Saturday(BaseModel):
    r"""Saturday schedule"""

    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Day schedule enabled"""

    hours: Optional[List[TimeRangeDto]] = None
    r"""Hours"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["hours"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class SundayTypedDict(TypedDict):
    r"""Sunday schedule"""

    is_enabled: bool
    r"""Day schedule enabled"""
    hours: NotRequired[List[TimeRangeDtoTypedDict]]
    r"""Hours"""


class Sunday(BaseModel):
    r"""Sunday schedule"""

    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Day schedule enabled"""

    hours: Optional[List[TimeRangeDto]] = None
    r"""Hours"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["hours"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class WeeklyScheduleTypedDict(TypedDict):
    r"""Weekly schedule"""

    monday: NotRequired[MondayTypedDict]
    r"""Monday schedule"""
    tuesday: NotRequired[TuesdayTypedDict]
    r"""Tuesday schedule"""
    wednesday: NotRequired[WednesdayTypedDict]
    r"""Wednesday schedule"""
    thursday: NotRequired[ThursdayTypedDict]
    r"""Thursday schedule"""
    friday: NotRequired[FridayTypedDict]
    r"""Friday schedule"""
    saturday: NotRequired[SaturdayTypedDict]
    r"""Saturday schedule"""
    sunday: NotRequired[SundayTypedDict]
    r"""Sunday schedule"""


class WeeklySchedule(BaseModel):
    r"""Weekly schedule"""

    monday: Optional[Monday] = None
    r"""Monday schedule"""

    tuesday: Optional[Tuesday] = None
    r"""Tuesday schedule"""

    wednesday: Optional[Wednesday] = None
    r"""Wednesday schedule"""

    thursday: Optional[Thursday] = None
    r"""Thursday schedule"""

    friday: Optional[Friday] = None
    r"""Friday schedule"""

    saturday: Optional[Saturday] = None
    r"""Saturday schedule"""

    sunday: Optional[Sunday] = None
    r"""Sunday schedule"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "monday",
                "tuesday",
                "wednesday",
                "thursday",
                "friday",
                "saturday",
                "sunday",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ScheduleDtoTypedDict(TypedDict):
    is_enabled: bool
    r"""Schedule enabled"""
    weekly_schedule: NotRequired[WeeklyScheduleTypedDict]
    r"""Weekly schedule"""


class ScheduleDto(BaseModel):
    is_enabled: Annotated[bool, pydantic.Field(alias="isEnabled")]
    r"""Schedule enabled"""

    weekly_schedule: Annotated[
        Optional[WeeklySchedule], pydantic.Field(alias="weeklySchedule")
    ] = None
    r"""Weekly schedule"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["weeklySchedule"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Monday.model_rebuild()
except NameError:
    pass
try:
    Tuesday.model_rebuild()
except NameError:
    pass
try:
    Wednesday.model_rebuild()
except NameError:
    pass
try:
    Thursday.model_rebuild()
except NameError:
    pass
try:
    Friday.model_rebuild()
except NameError:
    pass
try:
    Saturday.model_rebuild()
except NameError:
    pass
try:
    Sunday.model_rebuild()
except NameError:
    pass
try:
    ScheduleDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .notificationtriggervariable import (
    NotificationTriggerVariable,
    NotificationTriggerVariableTypedDict,
)
from enum import Enum
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class NotificationTriggerDtoType(str, Enum):
    r"""Type of the trigger"""

    EVENT = "event"


class NotificationTriggerDtoTypedDict(TypedDict):
    type: NotificationTriggerDtoType
    r"""Type of the trigger"""
    identifier: str
    r"""Identifier of the trigger"""
    variables: List[NotificationTriggerVariableTypedDict]
    r"""Variables of the trigger"""
    subscriber_variables: NotRequired[List[NotificationTriggerVariableTypedDict]]
    r"""Subscriber variables of the trigger"""


class NotificationTriggerDto(BaseModel):
    type: NotificationTriggerDtoType
    r"""Type of the trigger"""

    identifier: str
    r"""Identifier of the trigger"""

    variables: List[NotificationTriggerVariable]
    r"""Variables of the trigger"""

    subscriber_variables: Annotated[
        Optional[List[NotificationTriggerVariable]],
        pydantic.Field(alias="subscriberVariables"),
    ] = None
    r"""Subscriber variables of the trigger"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["subscriberVariables"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    NotificationTriggerDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TopicSubscriberIdentifierDtoTypedDict(TypedDict):
    identifier: str
    r"""Unique identifier for this subscription"""
    subscriber_id: str
    r"""The subscriber ID"""
    name: NotRequired[str]
    r"""The name of the subscription"""


class TopicSubscriberIdentifierDto(BaseModel):
    identifier: str
    r"""Unique identifier for this subscription"""

    subscriber_id: Annotated[str, pydantic.Field(alias="subscriberId")]
    r"""The subscriber ID"""

    name: Optional[str] = None
    r"""The name of the subscription"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["name"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    TopicSubscriberIdentifierDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .channelpreferencedto import ChannelPreferenceDto, ChannelPreferenceDtoTypedDict
from .workflowpreferencedto import WorkflowPreferenceDto, WorkflowPreferenceDtoTypedDict
from .workflowpreferencesdto import (
    WorkflowPreferencesDto,
    WorkflowPreferencesDtoTypedDict,
)
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict
from typing_extensions import NotRequired, TypedDict


WorkflowPreferencesResponseDtoAllTypedDict = WorkflowPreferenceDtoTypedDict
r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""


WorkflowPreferencesResponseDtoAll = WorkflowPreferenceDto
r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""


class WorkflowPreferencesResponseDtoUserTypedDict(TypedDict):
    r"""User-specific workflow preferences"""

    all: WorkflowPreferencesResponseDtoAllTypedDict
    r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""
    channels: Dict[str, ChannelPreferenceDtoTypedDict]
    r"""Preferences for different communication channels"""


class WorkflowPreferencesResponseDtoUser(BaseModel):
    r"""User-specific workflow preferences"""

    all: WorkflowPreferencesResponseDtoAll
    r"""A preference for the workflow. The values specified here will be used if no preference is specified for a channel."""

    channels: Dict[str, ChannelPreferenceDto]
    r"""Preferences for different communication channels"""


class WorkflowPreferencesResponseDtoTypedDict(TypedDict):
    default: WorkflowPreferencesDtoTypedDict
    r"""Default workflow preferences"""
    user: NotRequired[Nullable[WorkflowPreferencesResponseDtoUserTypedDict]]
    r"""User-specific workflow preferences"""


class WorkflowPreferencesResponseDto(BaseModel):
    default: WorkflowPreferencesDto
    r"""Default workflow preferences"""

    user: OptionalNullable[WorkflowPreferencesResponseDtoUser] = UNSET
    r"""User-specific workflow preferences"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["user"])
        nullable_fields = set(["user"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class SyncActionEnum(str, Enum):
    r"""Sync action performed"""

    CREATED = "created"
    UPDATED = "updated"
    SKIPPED = "skipped"
    DELETED = "deleted"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .scheduledto import ScheduleDto, ScheduleDtoTypedDict
from .subscriberpreferencechannels import (
    SubscriberPreferenceChannels,
    SubscriberPreferenceChannelsTypedDict,
)
from novu_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class SubscriberGlobalPreferenceDtoTypedDict(TypedDict):
    enabled: bool
    r"""Whether notifications are enabled globally"""
    channels: SubscriberPreferenceChannelsTypedDict
    r"""Channel-specific preference settings"""
    schedule: NotRequired[ScheduleDtoTypedDict]
    r"""Subscriber schedule"""


class SubscriberGlobalPreferenceDto(BaseModel):
    enabled: bool
    r"""Whether notifications are enabled globally"""

    channels: SubscriberPreferenceChannels
    r"""Channel-specific preference settings"""

    schedule: Optional[ScheduleDto] = None
    r"""Subscriber schedule"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["schedule"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .bridgeconfigurationdto import (
    BridgeConfigurationDto,
    BridgeConfigurationDtoTypedDict,
)
from .inboundparsedomaindto import InBoundParseDomainDto, InBoundParseDomainDtoTypedDict
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UpdateEnvironmentRequestDtoTypedDict(TypedDict):
    name: NotRequired[str]
    identifier: NotRequired[str]
    parent_id: NotRequired[str]
    color: NotRequired[str]
    dns: NotRequired[InBoundParseDomainDtoTypedDict]
    bridge: NotRequired[BridgeConfigurationDtoTypedDict]


class UpdateEnvironmentRequestDto(BaseModel):
    name: Optional[str] = None

    identifier: Optional[str] = None

    parent_id: Annotated[Optional[str], pydantic.Field(alias="parentId")] = None

    color: Optional[str] = None

    dns: Optional[InBoundParseDomainDto] = None

    bridge: Optional[BridgeConfigurationDto] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["name", "identifier", "parentId", "color", "dns", "bridge"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    UpdateEnvironmentRequestDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptionpreferencedto import (
    SubscriptionPreferenceDto,
    SubscriptionPreferenceDtoTypedDict,
)
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SubscriptionDetailsResponseDtoTypedDict(TypedDict):
    id: str
    r"""The unique identifier of the subscription"""
    identifier: NotRequired[str]
    r"""The identifier of the subscription"""
    name: NotRequired[str]
    r"""The name of the subscription"""
    preferences: NotRequired[List[SubscriptionPreferenceDtoTypedDict]]
    r"""The preferences/rules for the subscription"""
    context_keys: NotRequired[List[str]]
    r"""Context keys that scope this subscription (e.g., tenant:org-a, project:proj-123)"""


class SubscriptionDetailsResponseDto(BaseModel):
    id: str
    r"""The unique identifier of the subscription"""

    identifier: Optional[str] = None
    r"""The identifier of the subscription"""

    name: Optional[str] = None
    r"""The name of the subscription"""

    preferences: Optional[List[SubscriptionPreferenceDto]] = None
    r"""The preferences/rules for the subscription"""

    context_keys: Annotated[
        Optional[List[str]], pydantic.Field(alias="contextKeys")
    ] = None
    r"""Context keys that scope this subscription (e.g., tenant:org-a, project:proj-123)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["identifier", "name", "preferences", "contextKeys"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    SubscriptionDetailsResponseDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DiffActionEnum(str, Enum):
    r"""Type of change"""

    ADDED = "added"
    MODIFIED = "modified"
    DELETED = "deleted"
    UNCHANGED = "unchanged"
    MOVED = "moved"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatstepupsertdto import ChatStepUpsertDto, ChatStepUpsertDtoTypedDict
from .customstepupsertdto import CustomStepUpsertDto, CustomStepUpsertDtoTypedDict
from .delaystepupsertdto import DelayStepUpsertDto, DelayStepUpsertDtoTypedDict
from .digeststepupsertdto import DigestStepUpsertDto, DigestStepUpsertDtoTypedDict
from .emailstepupsertdto import EmailStepUpsertDto, EmailStepUpsertDtoTypedDict
from .inappstepupsertdto import InAppStepUpsertDto, InAppStepUpsertDtoTypedDict
from .preferencesrequestdto import PreferencesRequestDto, PreferencesRequestDtoTypedDict
from .pushstepupsertdto import PushStepUpsertDto, PushStepUpsertDtoTypedDict
from .resourceoriginenum import ResourceOriginEnum
from .severitylevelenum import SeverityLevelEnum
from .smsstepupsertdto import SmsStepUpsertDto, SmsStepUpsertDtoTypedDict
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from novu_py.utils import get_discriminator
import pydantic
from pydantic import Discriminator, Tag, model_serializer
from typing import Any, Dict, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


UpdateWorkflowDtoStepsTypedDict = TypeAliasType(
    "UpdateWorkflowDtoStepsTypedDict",
    Union[
        InAppStepUpsertDtoTypedDict,
        EmailStepUpsertDtoTypedDict,
        SmsStepUpsertDtoTypedDict,
        PushStepUpsertDtoTypedDict,
        ChatStepUpsertDtoTypedDict,
        DelayStepUpsertDtoTypedDict,
        DigestStepUpsertDtoTypedDict,
        CustomStepUpsertDtoTypedDict,
    ],
)


UpdateWorkflowDtoSteps = Annotated[
    Union[
        Annotated[InAppStepUpsertDto, Tag("in_app")],
        Annotated[EmailStepUpsertDto, Tag("email")],
        Annotated[SmsStepUpsertDto, Tag("sms")],
        Annotated[PushStepUpsertDto, Tag("push")],
        Annotated[ChatStepUpsertDto, Tag("chat")],
        Annotated[DelayStepUpsertDto, Tag("delay")],
        Annotated[DigestStepUpsertDto, Tag("digest")],
        Annotated[CustomStepUpsertDto, Tag("custom")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class UpdateWorkflowDtoTypedDict(TypedDict):
    name: str
    r"""Name of the workflow"""
    steps: List[UpdateWorkflowDtoStepsTypedDict]
    r"""Steps of the workflow"""
    preferences: PreferencesRequestDtoTypedDict
    r"""Workflow preferences"""
    origin: ResourceOriginEnum
    r"""Origin of the layout"""
    description: NotRequired[str]
    r"""Description of the workflow"""
    tags: NotRequired[List[str]]
    r"""Tags associated with the workflow"""
    active: NotRequired[bool]
    r"""Whether the workflow is active"""
    validate_payload: NotRequired[bool]
    r"""Enable or disable payload schema validation"""
    payload_schema: NotRequired[Nullable[Dict[str, Any]]]
    r"""The payload JSON Schema for the workflow"""
    is_translation_enabled: NotRequired[bool]
    r"""Enable or disable translations for this workflow"""
    workflow_id: NotRequired[str]
    r"""Workflow ID (allowed only for code-first workflows)"""
    severity: NotRequired[SeverityLevelEnum]
    r"""Severity of the workflow"""


class UpdateWorkflowDto(BaseModel):
    name: str
    r"""Name of the workflow"""

    steps: List[UpdateWorkflowDtoSteps]
    r"""Steps of the workflow"""

    preferences: PreferencesRequestDto
    r"""Workflow preferences"""

    origin: ResourceOriginEnum
    r"""Origin of the layout"""

    description: Optional[str] = None
    r"""Description of the workflow"""

    tags: Optional[List[str]] = None
    r"""Tags associated with the workflow"""

    active: Optional[bool] = False
    r"""Whether the workflow is active"""

    validate_payload: Annotated[
        Optional[bool], pydantic.Field(alias="validatePayload")
    ] = None
    r"""Enable or disable payload schema validation"""

    payload_schema: Annotated[
        OptionalNullable[Dict[str, Any]], pydantic.Field(alias="payloadSchema")
    ] = UNSET
    r"""The payload JSON Schema for the workflow"""

    is_translation_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="isTranslationEnabled")
    ] = False
    r"""Enable or disable translations for this workflow"""

    workflow_id: Annotated[Optional[str], pydantic.Field(alias="workflowId")] = None
    r"""Workflow ID (allowed only for code-first workflows)"""

    severity: Optional[SeverityLevelEnum] = None
    r"""Severity of the workflow"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "description",
                "tags",
                "active",
                "validatePayload",
                "payloadSchema",
                "isTranslationEnabled",
                "workflowId",
                "severity",
            ]
        )
        nullable_fields = set(["payloadSchema"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    UpdateWorkflowDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, List, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


FiveTypedDict = TypeAliasType("FiveTypedDict", Union[str, float, bool, Dict[str, Any]])


Five = TypeAliasType("Five", Union[str, float, bool, Dict[str, Any]])


class FourTypedDict(TypedDict):
    pass


class Four(BaseModel):
    pass


ValueTypedDict = TypeAliasType(
    "ValueTypedDict",
    Union[FourTypedDict, str, float, bool, List[Nullable[FiveTypedDict]]],
)
r"""Value that failed validation"""


Value = TypeAliasType("Value", Union[Four, str, float, bool, List[Nullable[Five]]])
r"""Value that failed validation"""


class ConstraintValidationTypedDict(TypedDict):
    messages: List[str]
    r"""List of validation error messages"""
    value: NotRequired[Nullable[ValueTypedDict]]
    r"""Value that failed validation"""


class ConstraintValidation(BaseModel):
    messages: List[str]
    r"""List of validation error messages"""

    value: OptionalNullable[Value] = UNSET
    r"""Value that failed validation"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["value"])
        nullable_fields = set(["value"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


FeedIdentifierTypedDict = TypeAliasType(
    "FeedIdentifierTypedDict", Union[str, List[str]]
)
r"""Optional feed identifier or array of feed identifiers"""


FeedIdentifier = TypeAliasType("FeedIdentifier", Union[str, List[str]])
r"""Optional feed identifier or array of feed identifiers"""


class MarkAs(str, Enum):
    r"""Mark all subscriber messages as read, unread, seen or unseen"""

    READ = "read"
    SEEN = "seen"
    UNREAD = "unread"
    UNSEEN = "unseen"


class MarkAllMessageAsRequestDtoTypedDict(TypedDict):
    mark_as: MarkAs
    r"""Mark all subscriber messages as read, unread, seen or unseen"""
    feed_identifier: NotRequired[FeedIdentifierTypedDict]
    r"""Optional feed identifier or array of feed identifiers"""


class MarkAllMessageAsRequestDto(BaseModel):
    mark_as: Annotated[MarkAs, pydantic.Field(alias="markAs")]
    r"""Mark all subscriber messages as read, unread, seen or unseen"""

    feed_identifier: Annotated[
        Optional[FeedIdentifier], pydantic.Field(alias="feedIdentifier")
    ] = None
    r"""Optional feed identifier or array of feed identifiers"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["feedIdentifier"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    MarkAllMessageAsRequestDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GroupPreferenceFilterDetailsDtoTypedDict(TypedDict):
    workflow_ids: NotRequired[List[str]]
    r"""List of workflow identifiers"""
    tags: NotRequired[List[str]]
    r"""List of tags"""


class GroupPreferenceFilterDetailsDto(BaseModel):
    workflow_ids: Annotated[
        Optional[List[str]], pydantic.Field(alias="workflowIds")
    ] = None
    r"""List of workflow identifiers"""

    tags: Optional[List[str]] = None
    r"""List of tags"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["workflowIds", "tags"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    GroupPreferenceFilterDetailsDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class APIKeyDtoTypedDict(TypedDict):
    key: str
    r"""API key"""
    user_id: str
    r"""User ID associated with the API key"""
    hash: NotRequired[str]
    r"""Hashed representation of the API key"""


class APIKeyDto(BaseModel):
    key: str
    r"""API key"""

    user_id: Annotated[str, pydantic.Field(alias="_userId")]
    r"""User ID associated with the API key"""

    hash: Optional[str] = None
    r"""Hashed representation of the API key"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["hash"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    APIKeyDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .stepissuesdto import StepIssuesDto, StepIssuesDtoTypedDict
from .steptypeenum import StepTypeEnum
from novu_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class StepListResponseDtoTypedDict(TypedDict):
    slug: str
    r"""Slug of the step"""
    type: StepTypeEnum
    r"""Type of the step"""
    issues: NotRequired[StepIssuesDtoTypedDict]
    r"""Issues associated with the step"""


class StepListResponseDto(BaseModel):
    slug: str
    r"""Slug of the step"""

    type: StepTypeEnum
    r"""Type of the step"""

    issues: Optional[StepIssuesDto] = None
    r"""Issues associated with the step"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["issues"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict
from typing_extensions import Annotated, NotRequired, TypedDict


class PatchSubscriberRequestDtoTypedDict(TypedDict):
    first_name: NotRequired[Nullable[str]]
    r"""First name of the subscriber"""
    last_name: NotRequired[Nullable[str]]
    r"""Last name of the subscriber"""
    email: NotRequired[Nullable[str]]
    r"""Email address of the subscriber"""
    phone: NotRequired[Nullable[str]]
    r"""Phone number of the subscriber"""
    avatar: NotRequired[Nullable[str]]
    r"""Avatar URL or identifier"""
    locale: NotRequired[Nullable[str]]
    r"""Locale of the subscriber"""
    timezone: NotRequired[Nullable[str]]
    r"""Timezone of the subscriber"""
    data: NotRequired[Nullable[Dict[str, Any]]]
    r"""Additional custom data associated with the subscriber"""


class PatchSubscriberRequestDto(BaseModel):
    first_name: Annotated[OptionalNullable[str], pydantic.Field(alias="firstName")] = (
        UNSET
    )
    r"""First name of the subscriber"""

    last_name: Annotated[OptionalNullable[str], pydantic.Field(alias="lastName")] = (
        UNSET
    )
    r"""Last name of the subscriber"""

    email: OptionalNullable[str] = UNSET
    r"""Email address of the subscriber"""

    phone: OptionalNullable[str] = UNSET
    r"""Phone number of the subscriber"""

    avatar: OptionalNullable[str] = UNSET
    r"""Avatar URL or identifier"""

    locale: OptionalNullable[str] = UNSET
    r"""Locale of the subscriber"""

    timezone: OptionalNullable[str] = UNSET
    r"""Timezone of the subscriber"""

    data: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Additional custom data associated with the subscriber"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "firstName",
                "lastName",
                "email",
                "phone",
                "avatar",
                "locale",
                "timezone",
                "data",
            ]
        )
        nullable_fields = set(
            [
                "firstName",
                "lastName",
                "email",
                "phone",
                "avatar",
                "locale",
                "timezone",
                "data",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    PatchSubscriberRequestDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .diffsummarydto import DiffSummaryDto, DiffSummaryDtoTypedDict
from .resourcedependencydto import ResourceDependencyDto, ResourceDependencyDtoTypedDict
from .resourcediffdto import ResourceDiffDto, ResourceDiffDtoTypedDict
from .resourcetypeenum import ResourceTypeEnum
from datetime import datetime
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ResourceDiffResultDtoSourceResourceUpdatedByTypedDict(TypedDict):
    r"""User who last updated the resource"""

    id: str
    r"""User ID"""
    first_name: str
    r"""User first name"""
    last_name: NotRequired[Nullable[str]]
    r"""User last name"""
    external_id: NotRequired[str]
    r"""User external ID"""


class ResourceDiffResultDtoSourceResourceUpdatedBy(BaseModel):
    r"""User who last updated the resource"""

    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""User ID"""

    first_name: Annotated[str, pydantic.Field(alias="firstName")]
    r"""User first name"""

    last_name: Annotated[OptionalNullable[str], pydantic.Field(alias="lastName")] = (
        UNSET
    )
    r"""User last name"""

    external_id: Annotated[Optional[str], pydantic.Field(alias="externalId")] = None
    r"""User external ID"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["lastName", "externalId"])
        nullable_fields = set(["lastName"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class SourceResourceTypedDict(TypedDict):
    r"""Source resource information"""

    id: NotRequired[Nullable[str]]
    r"""Resource ID (workflow ID or step ID)"""
    name: NotRequired[Nullable[str]]
    r"""Resource name (workflow name or step name)"""
    updated_by: NotRequired[
        Nullable[ResourceDiffResultDtoSourceResourceUpdatedByTypedDict]
    ]
    r"""User who last updated the resource"""
    updated_at: NotRequired[Nullable[datetime]]
    r"""When the resource was last updated"""


class SourceResource(BaseModel):
    r"""Source resource information"""

    id: OptionalNullable[str] = UNSET
    r"""Resource ID (workflow ID or step ID)"""

    name: OptionalNullable[str] = UNSET
    r"""Resource name (workflow name or step name)"""

    updated_by: Annotated[
        OptionalNullable[ResourceDiffResultDtoSourceResourceUpdatedBy],
        pydantic.Field(alias="updatedBy"),
    ] = UNSET
    r"""User who last updated the resource"""

    updated_at: Annotated[
        OptionalNullable[datetime], pydantic.Field(alias="updatedAt")
    ] = UNSET
    r"""When the resource was last updated"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["id", "name", "updatedBy", "updatedAt"])
        nullable_fields = set(["id", "name", "updatedBy", "updatedAt"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class ResourceDiffResultDtoUpdatedByTypedDict(TypedDict):
    r"""User who last updated the resource"""

    id: str
    r"""User ID"""
    first_name: str
    r"""User first name"""
    last_name: NotRequired[Nullable[str]]
    r"""User last name"""
    external_id: NotRequired[str]
    r"""User external ID"""


class ResourceDiffResultDtoUpdatedBy(BaseModel):
    r"""User who last updated the resource"""

    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""User ID"""

    first_name: Annotated[str, pydantic.Field(alias="firstName")]
    r"""User first name"""

    last_name: Annotated[OptionalNullable[str], pydantic.Field(alias="lastName")] = (
        UNSET
    )
    r"""User last name"""

    external_id: Annotated[Optional[str], pydantic.Field(alias="externalId")] = None
    r"""User external ID"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["lastName", "externalId"])
        nullable_fields = set(["lastName"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class TargetResourceTypedDict(TypedDict):
    r"""Target resource information"""

    id: NotRequired[Nullable[str]]
    r"""Resource ID (workflow ID or step ID)"""
    name: NotRequired[Nullable[str]]
    r"""Resource name (workflow name or step name)"""
    updated_by: NotRequired[Nullable[ResourceDiffResultDtoUpdatedByTypedDict]]
    r"""User who last updated the resource"""
    updated_at: NotRequired[Nullable[datetime]]
    r"""When the resource was last updated"""


class TargetResource(BaseModel):
    r"""Target resource information"""

    id: OptionalNullable[str] = UNSET
    r"""Resource ID (workflow ID or step ID)"""

    name: OptionalNullable[str] = UNSET
    r"""Resource name (workflow name or step name)"""

    updated_by: Annotated[
        OptionalNullable[ResourceDiffResultDtoUpdatedBy],
        pydantic.Field(alias="updatedBy"),
    ] = UNSET
    r"""User who last updated the resource"""

    updated_at: Annotated[
        OptionalNullable[datetime], pydantic.Field(alias="updatedAt")
    ] = UNSET
    r"""When the resource was last updated"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["id", "name", "updatedBy", "updatedAt"])
        nullable_fields = set(["id", "name", "updatedBy", "updatedAt"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


class ResourceDiffResultDtoTypedDict(TypedDict):
    resource_type: ResourceTypeEnum
    r"""Type of the layout"""
    changes: List[ResourceDiffDtoTypedDict]
    r"""List of specific changes for this resource"""
    summary: DiffSummaryDtoTypedDict
    r"""Summary of changes for this resource"""
    source_resource: NotRequired[Nullable[SourceResourceTypedDict]]
    r"""Source resource information"""
    target_resource: NotRequired[Nullable[TargetResourceTypedDict]]
    r"""Target resource information"""
    dependencies: NotRequired[List[ResourceDependencyDtoTypedDict]]
    r"""Dependencies that affect this resource"""


class ResourceDiffResultDto(BaseModel):
    resource_type: Annotated[ResourceTypeEnum, pydantic.Field(alias="resourceType")]
    r"""Type of the layout"""

    changes: List[ResourceDiffDto]
    r"""List of specific changes for this resource"""

    summary: DiffSummaryDto
    r"""Summary of changes for this resource"""

    source_resource: Annotated[
        OptionalNullable[SourceResource], pydantic.Field(alias="sourceResource")
    ] = UNSET
    r"""Source resource information"""

    target_resource: Annotated[
        OptionalNullable[TargetResource], pydantic.Field(alias="targetResource")
    ] = UNSET
    r"""Target resource information"""

    dependencies: Optional[List[ResourceDependencyDto]] = None
    r"""Dependencies that affect this resource"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["sourceResource", "targetResource", "dependencies"])
        nullable_fields = set(["sourceResource", "targetResource"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    ResourceDiffResultDtoSourceResourceUpdatedBy.model_rebuild()
except NameError:
    pass
try:
    SourceResource.model_rebuild()
except NameError:
    pass
try:
    ResourceDiffResultDtoUpdatedBy.model_rebuild()
except NameError:
    pass
try:
    TargetResource.model_rebuild()
except NameError:
    pass
try:
    ResourceDiffResultDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DependencyReasonEnum(str, Enum):
    r"""Reason for the dependency"""

    LAYOUT_REQUIRED_FOR_WORKFLOW = "LAYOUT_REQUIRED_FOR_WORKFLOW"
    LAYOUT_EXISTS_IN_TARGET = "LAYOUT_EXISTS_IN_TARGET"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .slackchannelendpointdto import (
    SlackChannelEndpointDto,
    SlackChannelEndpointDtoTypedDict,
)
from enum import Enum
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class CreateSlackChannelEndpointDtoContext2TypedDict(TypedDict):
    r"""Rich context object with id and optional data"""

    id: str
    data: NotRequired[Dict[str, Any]]
    r"""Optional additional context data"""


class CreateSlackChannelEndpointDtoContext2(BaseModel):
    r"""Rich context object with id and optional data"""

    id: str

    data: Optional[Dict[str, Any]] = None
    r"""Optional additional context data"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["data"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


CreateSlackChannelEndpointDtoContextTypedDict = TypeAliasType(
    "CreateSlackChannelEndpointDtoContextTypedDict",
    Union[CreateSlackChannelEndpointDtoContext2TypedDict, str],
)


CreateSlackChannelEndpointDtoContext = TypeAliasType(
    "CreateSlackChannelEndpointDtoContext",
    Union[CreateSlackChannelEndpointDtoContext2, str],
)


class CreateSlackChannelEndpointDtoType(str, Enum):
    r"""Type of channel endpoint"""

    SLACK_CHANNEL = "slack_channel"


class CreateSlackChannelEndpointDtoTypedDict(TypedDict):
    subscriber_id: str
    r"""The subscriber ID to which the channel endpoint is linked"""
    integration_identifier: str
    r"""The identifier of the integration to use for this channel endpoint."""
    type: CreateSlackChannelEndpointDtoType
    r"""Type of channel endpoint"""
    endpoint: SlackChannelEndpointDtoTypedDict
    r"""Slack channel endpoint data"""
    identifier: NotRequired[str]
    r"""The unique identifier for the channel endpoint. If not provided, one will be generated automatically."""
    context: NotRequired[Dict[str, CreateSlackChannelEndpointDtoContextTypedDict]]
    connection_identifier: NotRequired[str]
    r"""The identifier of the channel connection to use for this channel endpoint."""


class CreateSlackChannelEndpointDto(BaseModel):
    subscriber_id: Annotated[str, pydantic.Field(alias="subscriberId")]
    r"""The subscriber ID to which the channel endpoint is linked"""

    integration_identifier: Annotated[
        str, pydantic.Field(alias="integrationIdentifier")
    ]
    r"""The identifier of the integration to use for this channel endpoint."""

    type: CreateSlackChannelEndpointDtoType
    r"""Type of channel endpoint"""

    endpoint: SlackChannelEndpointDto
    r"""Slack channel endpoint data"""

    identifier: Optional[str] = None
    r"""The unique identifier for the channel endpoint. If not provided, one will be generated automatically."""

    context: Optional[Dict[str, CreateSlackChannelEndpointDtoContext]] = None

    connection_identifier: Annotated[
        Optional[str], pydantic.Field(alias="connectionIdentifier")
    ] = None
    r"""The identifier of the channel connection to use for this channel endpoint."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["identifier", "context", "connectionIdentifier"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreateSlackChannelEndpointDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ImportMasterJSONResponseDtoTypedDict(TypedDict):
    success: bool
    r"""Overall success status of the import operation"""
    message: str
    r"""Human-readable message describing the import result"""
    successful: NotRequired[List[str]]
    r"""List of resource IDs that were successfully imported"""
    failed: NotRequired[List[str]]
    r"""List of resource IDs that failed to import"""


class ImportMasterJSONResponseDto(BaseModel):
    success: bool
    r"""Overall success status of the import operation"""

    message: str
    r"""Human-readable message describing the import result"""

    successful: Optional[List[str]] = None
    r"""List of resource IDs that were successfully imported"""

    failed: Optional[List[str]] = None
    r"""List of resource IDs that failed to import"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["successful", "failed"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class EnvironmentDiffSummaryDtoTypedDict(TypedDict):
    total_entities: float
    r"""Total number of entities compared"""
    total_changes: float
    r"""Total number of changes detected"""
    has_changes: bool
    r"""Whether any changes were detected"""


class EnvironmentDiffSummaryDto(BaseModel):
    total_entities: Annotated[float, pydantic.Field(alias="totalEntities")]
    r"""Total number of entities compared"""

    total_changes: Annotated[float, pydantic.Field(alias="totalChanges")]
    r"""Total number of changes detected"""

    has_changes: Annotated[bool, pydantic.Field(alias="hasChanges")]
    r"""Whether any changes were detected"""


try:
    EnvironmentDiffSummaryDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class NotificationGroupTypedDict(TypedDict):
    name: str
    environment_id: str
    organization_id: str
    id: NotRequired[str]
    parent_id: NotRequired[str]


class NotificationGroup(BaseModel):
    name: str

    environment_id: Annotated[str, pydantic.Field(alias="_environmentId")]

    organization_id: Annotated[str, pydantic.Field(alias="_organizationId")]

    id: Annotated[Optional[str], pydantic.Field(alias="_id")] = None

    parent_id: Annotated[Optional[str], pydantic.Field(alias="_parentId")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["_id", "_parentId"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    NotificationGroup.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .patchpreferencechannelsdto import (
    PatchPreferenceChannelsDto,
    PatchPreferenceChannelsDtoTypedDict,
)
from .scheduledto import ScheduleDto, ScheduleDtoTypedDict
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class TwoTypedDict(TypedDict):
    r"""Rich context object with id and optional data"""

    id: str
    data: NotRequired[Dict[str, Any]]
    r"""Optional additional context data"""


class Two(BaseModel):
    r"""Rich context object with id and optional data"""

    id: str

    data: Optional[Dict[str, Any]] = None
    r"""Optional additional context data"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["data"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


ContextTypedDict = TypeAliasType("ContextTypedDict", Union[TwoTypedDict, str])


Context = TypeAliasType("Context", Union[Two, str])


class PatchSubscriberPreferencesDtoTypedDict(TypedDict):
    channels: NotRequired[PatchPreferenceChannelsDtoTypedDict]
    r"""Channel-specific preference settings"""
    workflow_id: NotRequired[str]
    r"""Workflow internal _id, identifier or slug. If provided, update workflow specific preferences, otherwise update global preferences"""
    schedule: NotRequired[ScheduleDtoTypedDict]
    r"""Subscriber schedule"""
    context: NotRequired[Dict[str, ContextTypedDict]]


class PatchSubscriberPreferencesDto(BaseModel):
    channels: Optional[PatchPreferenceChannelsDto] = None
    r"""Channel-specific preference settings"""

    workflow_id: Annotated[Optional[str], pydantic.Field(alias="workflowId")] = None
    r"""Workflow internal _id, identifier or slug. If provided, update workflow specific preferences, otherwise update global preferences"""

    schedule: Optional[ScheduleDto] = None
    r"""Subscriber schedule"""

    context: Optional[Dict[str, Context]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["channels", "workflowId", "schedule", "context"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    PatchSubscriberPreferencesDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .triggerrecipientstypeenum import TriggerRecipientsTypeEnum
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TopicPayloadDtoTypedDict(TypedDict):
    topic_key: str
    type: TriggerRecipientsTypeEnum
    exclude: NotRequired[List[str]]
    r"""Optional array of subscriber IDs to exclude from the topic trigger"""


class TopicPayloadDto(BaseModel):
    topic_key: Annotated[str, pydantic.Field(alias="topicKey")]

    type: TriggerRecipientsTypeEnum

    exclude: Optional[List[str]] = None
    r"""Optional array of subscriber IDs to exclude from the topic trigger"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["exclude"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    TopicPayloadDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import importlib.metadata

__title__: str = "novu-py"
__version__: str = "3.14.1"
__openapi_doc_version__: str = "3.14.0"
__gen_version__: str = "2.846.1"
__user_agent__: str = "speakeasy-sdk/python 3.14.1 2.846.1 3.14.0 novu-py"

try:
    if __package__ is not None:
        __version__ = importlib.metadata.version(__package__)
except importlib.metadata.PackageNotFoundError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class MarkMessageActionAsSeenDtoStatus(str, Enum):
    r"""Message action status"""

    PENDING = "pending"
    DONE = "done"


class MarkMessageActionAsSeenDtoPayloadTypedDict(TypedDict):
    r"""Message action payload"""


class MarkMessageActionAsSeenDtoPayload(BaseModel):
    r"""Message action payload"""


class MarkMessageActionAsSeenDtoTypedDict(TypedDict):
    status: MarkMessageActionAsSeenDtoStatus
    r"""Message action status"""
    payload: NotRequired[MarkMessageActionAsSeenDtoPayloadTypedDict]
    r"""Message action payload"""


class MarkMessageActionAsSeenDto(BaseModel):
    status: MarkMessageActionAsSeenDtoStatus
    r"""Message action status"""

    payload: Optional[MarkMessageActionAsSeenDtoPayload] = None
    r"""Message action payload"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["payload"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class CreateContextRequestDtoTypedDict(TypedDict):
    type: str
    r"""Context type (e.g., tenant, app, workspace). Must be lowercase alphanumeric with optional separators."""
    id: str
    r"""Unique identifier for this context. Must be lowercase alphanumeric with optional separators."""
    data: NotRequired[Dict[str, Any]]
    r"""Optional custom data to associate with this context."""


class CreateContextRequestDto(BaseModel):
    type: str
    r"""Context type (e.g., tenant, app, workspace). Must be lowercase alphanumeric with optional separators."""

    id: str
    r"""Unique identifier for this context. Must be lowercase alphanumeric with optional separators."""

    data: Optional[Dict[str, Any]] = None
    r"""Optional custom data to associate with this context."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["data"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TopicResponseDtoTypedDict(TypedDict):
    id: str
    r"""The identifier of the topic"""
    key: str
    r"""The unique key of the topic"""
    name: NotRequired[str]
    r"""The name of the topic"""
    created_at: NotRequired[str]
    r"""The date the topic was created"""
    updated_at: NotRequired[str]
    r"""The date the topic was last updated"""


class TopicResponseDto(BaseModel):
    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""The identifier of the topic"""

    key: str
    r"""The unique key of the topic"""

    name: Optional[str] = None
    r"""The name of the topic"""

    created_at: Annotated[Optional[str], pydantic.Field(alias="createdAt")] = None
    r"""The date the topic was created"""

    updated_at: Annotated[Optional[str], pydantic.Field(alias="updatedAt")] = None
    r"""The date the topic was last updated"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["name", "createdAt", "updatedAt"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    TopicResponseDto.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .diffenvironmentrequestdto import (
    DiffEnvironmentRequestDto,
    DiffEnvironmentRequestDtoTypedDict,
)
from .diffenvironmentresponsedto import (
    DiffEnvironmentResponseDto,
    DiffEnvironmentResponseDtoTypedDict,
)
from novu_py.types import BaseModel, UNSET_SENTINEL
from novu_py.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EnvironmentsControllerDiffEnvironmentRequestTypedDict(TypedDict):
    target_environment_id: str
    r"""Target environment ID (MongoDB ObjectId) to compare against"""
    diff_environment_request_dto: DiffEnvironmentRequestDtoTypedDict
    r"""Diff request configuration"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class EnvironmentsControllerDiffEnvironmentRequest(BaseModel):
    target_environment_id: Annotated[
        str,
        pydantic.Field(alias="targetEnvironmentId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""Target environment ID (MongoDB ObjectId) to compare against"""

    diff_environment_request_dto: Annotated[
        DiffEnvironmentRequestDto,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
    r"""Diff request configuration"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["idempotency-key"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class EnvironmentsControllerDiffEnvironmentResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: DiffEnvironmentResponseDtoTypedDict


class EnvironmentsControllerDiffEnvironmentResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: DiffEnvironmentResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .actortypeenum import ActorTypeEnum
from novu_py.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import TypedDict


class ActorFeedItemDtoTypedDict(TypedDict):
    data: Nullable[str]
    r"""The data associated with the actor, can be null if not applicable."""
    type: ActorTypeEnum
    r"""The type of the actor, indicating the role in the notification process."""


class ActorFeedItemDto(BaseModel):
    data: Nullable[str]
    r"""The data associated with the actor, can be null if not applicable."""

    type: ActorTypeEnum
    r"""The type of the actor, indicating the role in the notification process."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .uicomponentenum import UIComponentEnum
from novu_py import models
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import field_serializer, model_serializer
from typing import Any, Dict, List, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


Placeholder5TypedDict = TypeAliasType(
    "Placeholder5TypedDict", Union[str, float, bool, Dict[str, Any]]
)


Placeholder5 = TypeAliasType("Placeholder5", Union[str, float, bool, Dict[str, Any]])


PlaceholderTypedDict = TypeAliasType(
    "PlaceholderTypedDict",
    Union[str, float, bool, Dict[str, Any], List[Placeholder5TypedDict]],
)
r"""Placeholder for the UI Schema Property"""


Placeholder = TypeAliasType(
    "Placeholder", Union[str, float, bool, Dict[str, Any], List[Placeholder5]]
)
r"""Placeholder for the UI Schema Property"""


class UISchemaPropertyTypedDict(TypedDict):
    component: UIComponentEnum
    r"""Component type for the UI Schema Property"""
    placeholder: NotRequired[Nullable[PlaceholderTypedDict]]
    r"""Placeholder for the UI Schema Property"""
    properties: NotRequired[Dict[str, UISchemaPropertyTypedDict]]
    r"""Properties of the UI Schema"""


class UISchemaProperty(BaseModel):
    component: UIComponentEnum
    r"""Component type for the UI Schema Property"""

    placeholder: OptionalNullable[Placeholder] = UNSET
    r"""Placeholder for the UI Schema Property"""

    properties: Optional[Dict[str, UISchemaProperty]] = None
    r"""Properties of the UI Schema"""

    @field_serializer("component")
    def serialize_component(self, value):
        if isinstance(value, str):
            try:
                return models.UIComponentEnum(value)
            except ValueError:
                return value
        return value

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["placeholder", "properties"])
        nullable_fields = set(["placeholder"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriberdto import SubscriberDto, SubscriberDtoTypedDict
from .topicresponsedto import TopicResponseDto, TopicResponseDtoTypedDict
from novu_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TopicSubscriptionResponseDtoTypedDict(TypedDict):
    id: str
    r"""The identifier of the subscription"""
    identifier: str
    r"""The identifier of the subscription"""
    created_at: str
    r"""The date and time the subscription was created"""
    topic: TopicResponseDtoTypedDict
    r"""Topic information"""
    subscriber: SubscriberDtoTypedDict
    r"""Subscriber information"""
    context_keys: NotRequired[List[str]]
    r"""Context keys that scope this subscription (e.g., tenant:org-a, project:proj-123)"""


class TopicSubscriptionResponseDto(BaseModel):
    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""The identifier of the subscription"""

    identifier: str
    r"""The identifier of the subscription"""

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""The date and time the subscription was created"""

    topic: TopicResponseDto
    r"""Topic information"""

    subscriber: SubscriberDto
    r"""Subscriber information"""

    context_keys: Annotated[
        Optional[List[str]], pydantic.Field(alias="contextKeys")
    ] = None
    r"""Context keys that scope this subscription (e.g., tenant:org-a, project:proj-123)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["contextKeys"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    TopicSubscriptionResponseDto.model_rebuild()
except NameError:
    pass
